@@ -79,12 +79,17 @@ macro_rules! build_tail_array {
             let mut index = $index;
             let mut iter = ::core::iter::Iterator::inspect(&mut $iter, |_| {index += 1;});
 
-            match (move || Some([$($(
-                {
-                    stringify!($pattern);
-                    ::core::iter::Iterator::next(&mut iter)?
-                },
-            )+)?]))() {
+            match loop {
+                break ::core::option::Option::Some([$($(
+                    {
+                        stringify!($pattern);
+                        match ::core::iter::Iterator::next(&mut iter) {
+                            ::core::option::Option::Some(item) => item,
+                            ::core::option::Option::None => break ::core::option::Option::None,
+                        }
+                    },
+                )+)?])
+            } {
                 ::core::option::Option::Some(array) => array,
                 ::core::option::Option::None => $crate::assertion_failure!(
                     "iterable was too short",
@@ -349,6 +354,15 @@ mod tests {
         assert_eq!(c, 3);
     }
 
+    #[test]
+    fn nameless_temporary_borrow() {
+        fn make_vec() -> Vec<u32> {
+            vec![2, 3, 4]
+        }
+
+        assert_matches!(make_vec().as_slice(), [2, 3, 4]);
+    }
+
     #[test]
     fn iter_failure_mismatch() {
         let data = vec![None, Some(3), None];
